
/****************************************************************************/
/*                                                                          */
/*  Create and check the request / reply pair                               */
/*                                                                          */
/****************************************************************************/

#include <stdio.h>
#include <unistd.h>
#include <time.h>

#include <sys/utsname.h>
#include <sys/types.h>

#include <idsa_internal.h>

#define IDSA_MAGIC_REQUEST 0x02da	/* report request */
#define IDSA_MAGIC_REPLY   0x03da	/* reply */

/****************************************************************************/
/* Does       : Fill in reasonable defaults for a request event             */

int idsa_request_default(IDSA_EVENT * e, char *service, char *scheme,
			 char *name)
{
  int result = 0;

  IDSA_UNIT *u;
  unsigned int count, i;

  struct utsname ut;

  /* format event and create reserved fields */
  idsa_event_clear(e, IDSA_MAGIC_REQUEST);

  /* fill in the required fields */
  count = idsa_required_count();
  for (i = 0; i < count; i++) {
#ifdef TRACE
    fprintf(stderr, "idsa_request_default(): namebynumber(i)=<%s>\n",
	    idsa_reserved_namebynumber(i));
#endif
    u =
	idsa_event_scanappend(e, idsa_reserved_namebynumber(i),
			      idsa_reserved_typebynumber(i), NULL);
    if (u) {
#ifdef TRACE
      fprintf(stderr,
	      "idsa_request_default(): set up reserved <%s> to <%p>\n",
	      idsa_unit_name(u), u);
#endif
    } else {
      result++;
    }
  }

  result +=
      idsa_risks(e, 0, IDSA_RISKDEFAULT, IDSA_RISKDEFAULT,
		 IDSA_RISKDEFAULT);

  /* put sensible defaults into the required fields */
  result += idsa_service(e, service);
  result += idsa_scheme(e, scheme);
  result += idsa_name(e, name);
  result += idsa_host(e, uname(&ut) ? "localhost" : ut.nodename);
  result += idsa_uid(e, getuid());
  result += idsa_gid(e, getgid());
  result += idsa_pid(e, getpid());
  result += idsa_time(e, time(NULL));

#ifdef TRACE
  idsa_event_dump(e, stderr);
#endif

  return result;
}

/****************************************************************************/
/* Does       : Eyeballs event, attempts to overwrite as much as possible   */
/*              to make it consistent                                       */
/* Returns    : zero on success, nonzero otherwise                          */

int idsa_request_check(IDSA_EVENT * e)
{
  int result = 0;		/* return 0 if ok, nonzero otherwise */
  unsigned int count, i;
  IDSA_UNIT *u;

  if (e->e_magic != IDSA_MAGIC_REQUEST) {
#ifdef TRACE
    fprintf(stderr, "idsa_request_check(): bad magic\n");
#endif
    result++;
  }
  result += idsa_event_check(e);

  if (result == 0) {
    e->e_result = IDSA_L_ALLOW;
    count = idsa_required_count();
    if (count > e->e_count) {
#ifdef TRACE
      fprintf(stderr,
	      "idsa_request_check(): too few units in request event\n");
#endif
      result++;
    } else {			/* sufficient units */
      for (i = 0; i < count; i++) {
	u = idsa_event_unitbynumber(e, i);
	if (u) {
	  if (idsa_reserved_typebynumber(i) != idsa_unit_type(u)) {
	    result++;
	    i = e->e_count;
	  }
	} else {
	  result++;
	  i = e->e_count;
	}
      }
    }
  }
  return result;
}

/****************************************************************************/
/* Does       : Generates the default reply - note the default allow        */
/* Returns    : zero on success, nonzero otherwise                          */

int idsa_reply_default(IDSA_EVENT * e)
{
  int result = 0;

  idsa_event_clear(e, IDSA_MAGIC_REPLY);

  return result;
}

/****************************************************************************/
/* Does       : Establish if reply is valid                                 */
/* Returns    : zero on success, nonzero otherwise                          */

int idsa_reply_check(IDSA_EVENT * e)
{
  int result = 0;

  if (e->e_magic != IDSA_MAGIC_REPLY) {
    result++;
  } else {
    result += idsa_event_check(e);
  }

  return result;
}
