

/****************************************************************************/
/*                                                                          */
/*  Provide a list of required fields to create a new event, as well as     */
/*  well known optional units and units sent in the reply                   */
/*                                                                          */
/****************************************************************************/

#include <stdio.h>

#include <idsa_internal.h>

struct _idsa_reserved {
  char r_name[IDSA_M_NAME];
  unsigned int r_type;
};
typedef struct _idsa_reserved _IDSA_RESERVED;

static _IDSA_RESERVED _idsa_reserved_table[IDSA_M_RESERVED] = {
  [IDSA_N_UID] = {"uid", IDSA_T_UID},
  [IDSA_N_GID] = {"gid", IDSA_T_GID},
  [IDSA_N_PID] = {"pid", IDSA_T_PID},
  [IDSA_N_TIME] = {"time", IDSA_T_TIME},
  [IDSA_N_SERVICE] = {"service", IDSA_T_STRING},
  [IDSA_N_HOST] = {"host", IDSA_T_HOST},
  [IDSA_N_NAME] = {"name", IDSA_T_STRING},
  [IDSA_N_SCHEME] = {"scheme", IDSA_T_STRING},

  [IDSA_N_HONOUR] = {"honour", IDSA_T_FLAG},
  [IDSA_N_ARISK] = {"arisk", IDSA_T_RISK},
  [IDSA_N_CRISK] = {"crisk", IDSA_T_RISK},
  [IDSA_N_IRISK] = {"irisk", IDSA_T_RISK},

  /* things below here are not required, merely of known type */

  [IDSA_N_IGNORED] = {"ignored", IDSA_T_FLAG},
  [IDSA_N_JOB] = {"job", IDSA_T_INT},
  [IDSA_N_REASON] = {"reason", IDSA_T_STRING},
  [IDSA_N_COMMENT] = {"comment", IDSA_T_STRING},

  [IDSA_N_PREARISK] = {"pre/arisk", IDSA_T_RISK},
  [IDSA_N_PRECRISK] = {"pre/crisk", IDSA_T_RISK},
  [IDSA_N_PREIRISK] = {"pre/irisk", IDSA_T_RISK},

  [IDSA_N_FAILARISK] = {"fail/arisk", IDSA_T_RISK},
  [IDSA_N_FAILCRISK] = {"fail/crisk", IDSA_T_RISK},
  [IDSA_N_FAILIRISK] = {"fail/irisk", IDSA_T_RISK}
};

int idsa_required_count()
{
  return IDSA_M_REQUIRED;
}

unsigned int idsa_reserved_typebynumber(int n)
{
  if ((n > IDSA_M_RESERVED) || (n < 0)) {
    return IDSA_T_NULL;
  } else {
    return _idsa_reserved_table[n].r_type;
  }
}

char *idsa_reserved_namebynumber(int n)
{
  if ((n > IDSA_M_RESERVED) || (n < 0)) {
    return NULL;
  } else {
    return _idsa_reserved_table[n].r_name;
  }
}

unsigned int idsa_reserved_typebyname(char *n)
{
  int i;
  for (i = 0; i < IDSA_M_RESERVED; i++) {
    if (strcmp(_idsa_reserved_table[i].r_name, n) == 0) {
#ifdef TRACE
      fprintf(stderr, "idsa_reserved_lookup(): <%s> has type 0x%04x\n", n,
	      _idsa_reserved_table[i].r_type);
#endif
      return _idsa_reserved_table[i].r_type;
    }
  }
  return IDSA_T_NULL;
}

int idsa_reserved_numberbyname(char *n)
{
  int i;
  for (i = 0; i < IDSA_M_RESERVED; i++) {
    if (strcmp(_idsa_reserved_table[i].r_name, n) == 0) {
#ifdef TRACE
      fprintf(stderr, "idsa_reserved_index(): <%s> is reserved at <%d>\n",
	      n, i);
#endif
      return i;
    }
  }
  return i;
}
